@@ -1166,7 +1166,8 @@ static int imap_fetch_headers(int sock, struct query *ctl,int number,int *lenp)
 
 	/* try to recover for some responses */
 	if (!strncmp(buf, "* NO", 4) ||
-		!strncmp(buf, "* BAD", 5))
+		!strncmp(buf, "* BAD", 5) ||
+		strstr(buf, "FETCH ()"))
 	{
 	    return(PS_TRANSIENT);
 	}
@@ -1181,7 +1182,7 @@ static int imap_fetch_headers(int sock, struct query *ctl,int number,int *lenp)
 	/* an unexpected tagged response */
 	if (outlevel > O_SILENT)
 	    report(stderr, GT_("Incorrect FETCH response: %s.\n"), buf);
-	return(PS_ERROR);
+	return(PS_TRANSIENT);
     }
     return(ok);
 }
