@@ -1383,6 +1383,28 @@ process_headers:
 	return PS_SOCKET;
 }
 
+/** Convenience function factored out from readbody(): 
+ * send buffer \a buf via stuffline() and handle errors and progress.
+ * Store return value in \a *n, and return PS_IOERR for failure or
+ * PS_SUCCESS otherwise. */
+static int rb_send(struct query *ctl, char *buf, int *n)
+{
+    *n = stuffline(ctl, buf);
+
+    if (*n < 0)
+    {
+	report(stdout, GT_("error writing message text\n"));
+	release_sink(ctl);
+	return(PS_IOERR);
+    }
+    else if (want_progress())
+    {
+	fputc('*', stdout);
+	fflush(stdout);
+    }
+    return PS_SUCCESS;
+}
+
 int readbody(int sock, struct query *ctl, flag forward, int len)
 /** read and dispose of a message body presented on \a sock */
 /** \param ctl		query control record */
@@ -1435,7 +1457,15 @@ int readbody(int sock, struct query *ctl, flag forward, int len)
 	 * so we might end truncating messages prematurely.
 	 */
 	if (!protocol->delimited && linelen > len) {
+	    /* FIXME: HACK ALERT! This \r\n is only here to make sure the
+	     * \n\0 hunt works later on. The \n generated here was not
+	     * part of the original message!
+	     * The real fix will be to use buffer + length strings,
+	     * rather than 0-terminated C strings. */
+	    inbufp[len++] = '\r';
+	    inbufp[len++] = '\n';
 	    inbufp[len] = '\0';
+	    linelen = len;
 	}
 
 	len -= linelen;
@@ -1470,7 +1500,7 @@ int readbody(int sock, struct query *ctl, flag forward, int len)
 	/* ship out the text line */
 	if (forward && (!issoftline))
 	{
-	    int	n;
+	    int	n, err;
 	    inbufp = buf;
 
 	    /* guard against very long lines */
@@ -1478,22 +1508,31 @@ int readbody(int sock, struct query *ctl, flag forward, int len)
 	    buf[MSGBUFSIZE+2] = '\n';
 	    buf[MSGBUFSIZE+3] = '\0';
 
-	    n = stuffline(ctl, buf);
-
-	    if (n < 0)
-	    {
-		report(stdout, GT_("error writing message text\n"));
-		release_sink(ctl);
-		return(PS_IOERR);
-	    }
-	    else if (want_progress())
-	    {
-		fputc('*', stdout);
-		fflush(stdout);
-	    }
+	    err = rb_send(ctl, buf, &n);
+	    if (err != PS_SUCCESS)
+		return err;
 	}
     }
 
+    /* Flush buffer -- bug introduced by ESR on 1998-03-20 before
+     * release 4.4.1 when ESR did not sufficiently audit Henrik
+     * Storner's patch.
+     * Trouble reported in June 2011 by Lars Hecking, with
+     * text/html quoted-printable messages generated by
+     * Outlook/Exchange that got mutilated by fetchmail.
+     */
+    if (forward && issoftline)
+    {
+	int n;
+
+	/* force proper line termination */
+	inbufp[0] = '\r';
+	inbufp[1] = '\n';
+	inbufp[2] = '\0';
+
+	return rb_send(ctl, buf, &n);
+    }
+
     return(PS_SUCCESS);
 }
 
